;
; Lunokhod 1
; 
; Screen Layout:
;
;     Lines     Cycles      Description
;     -----------------------------------
;     40        3040        VSYNC/VBLANK    
;     10        760         Score
;     32        2432        Sky
;     130       9880        Playfield
;     20        1520        Radar
;     30        2280        Overscan
;   
; Flow:
; 
; * sysinit
; * non-sysinit
; * main loop
;  * START VERTICAL BLANK
;  * create shooting sounds
;  * create explosion sounds or background sounds
;  * update the animation offset counter
;  * initialize the position of the first sprite
;  * initialize the ball (used to hide hmove lines)
;  * adjust missile positions
;  * swap missile variables
;  * remove any stale debris
;  * adjust debris positions
;  * preload the first debris sprite before we get into the display kernel
;  * map the player into radar ram
;  * END VERTICAL BLANK
;  * KERNEL
;    * draw score
;    * draw sky
;    * draw mountains
;    * draw playfield
;    * draw hills
;    * draw player
;    * draw radar
;  * START OVERSCAN
;  * collision detection
;  * undo missile swapping
;  * process joystick input
;  * adjust player position variables
;  * adjust scroll offsets
;  * add new debris if we can
;  * clear the radar ram
;  * map debris objects into radar ram
;  * END OVERSCAN
;

    PROCESSOR 6502
    include "vcs.h"
    include "macro.h"

    SEG
    ORG $F000 ; set the ROM page
; 
; Variable declarations
;                         

score       equ  $80    ; 80-81
p0x         equ  $82    ; screen x position of player 0 [20,132]
p0s         equ  $83    ; x speed of player 0 [-4,4]
p0vxlo      equ  $84    ; player 0 virtual x position low byte
temp1lo     equ  $85    ; temporary 16
temp1hi     equ  $86    ; temporary 16
temp2lo     equ  $87    ; temporary 16
temp2hi     equ  $88    ; temporary 16
temp3       equ  $89    ;
radarColor  equ  $8a    ; 
;bits        equ  $8b    ; D7 = debounce bit,  
vxoffsetlo  equ  $8c    ;
bombState   equ  $8d    ; D0-D1 = number of bombs left, D7=debounce state
nexty       equ  $8e    ;
m0vxlo      equ  $8f    ; virtual x position of missile 0
m0ys        equ  $90    ; y position of missile 0
m1vxlo      equ  $91    ; virtual x position of missile 1
m1ys        equ  $92    ; y position of missile 1
m2vxlo      equ  $93    ; virtual x position of missile 2  
m2ys        equ  $94    ; y position of missile 2
m3vxlo      equ  $95    ; virtual x position of missile 3
m3ys        equ  $96    ; y position of missile 3
debrisvx     equ  $97    ; 97-9E
debrisinfo   equ  $9f    ; 9F-A6
debrisx      equ  $a7    ; A7-AE
debrisy      equ  $af    ; AF-B6
frameCount  equ  $b7    ; count the current frame. 
debrisCount  equ  $b8    ; number of current debris
scrollpos   equ  $b9    ;
p1dataptrlo equ  $ba    ;
p1dataptrhi equ  $bb    ;
wave        equ  $bc    ;
p1colorlo   equ  $bd    ;         
p1colorhi   equ  $be    ;   
spritecount equ  $bf    ; 
spritelist  equ  $c0    ; C0-C7        
sprite1lo   equ  $c8    ;       
sprite1hi   equ  $c9    ;         
sprite2lo   equ  $ca    ;         
sprite2hi   equ  $cb    ;         
sprite3lo   equ  $cc    ;         
sprite3hi   equ  $cd    ;         
sprite4lo   equ  $ce    ;         
sprite4hi   equ  $cf    ;         
sprite5lo   equ  $d0    ;         
sprite5hi   equ  $d1    ;         
sprite6lo   equ  $d2    ;         
sprite6hi   equ  $d3    ;         
sprite7lo   equ  $d4    ;         
sprite7hi   equ  $d5    ;         
sprite8lo   equ  $d6    ;         
sprite8hi   equ  $d7    ;  
audio2count equ  $D8    ;
shotcount   equ  $D9    ; 
mode        equ  $DA    ; 0=init,1-126=game over pause,127=trigger game over,128=game in progress,129-253=fade-in-game pause, 254=reset down, 255=reset up
radlevel    equ  $DB    ;
audio2type  equ  $DC    ;
random      equ  $DD    ;  
radarRam    equ  $DE    ; DE-FD  (two bytes left for stack!!)

;
; Constants
; 

MISSILE_FOLLOWS_SHIP = 0;

MAXSPEED     equ #2      ; the max speed of the sled
MINSPEED     equ #-2     ; the min speed of the sled
MINXPOS      equ #61-5      ; minimum screen positionof the sled (32)
MAXXPOS      equ #91+5    ; maximum screen position of the sled  (112)
POSDELTA     equ #MAXXPOS-MINXPOS        ; this must be equal to maxpos-minpos (80)
P0CENTER     equ #4     ; offset to center of player 0
MISSILESPEED equ #5
MISSILESIZE  equ #9

GAUGECOLOR   equ #$32
SCORECOLOR   equ #$0e
SKYCOLOR     equ #$00
MTNCOLOR     equ #$04
MTNCAPSCOLOR equ #$04
GRADIENT1    equ #$04
GRADIENT2    equ #$04
GRADIENT3    equ #$02
RADARCOLOR   equ #$04
RADARCOLOR2  equ #$48
GROUNDCOLOR  equ #$00
SNOWCOLOR    equ #$06
RAILCOLOR    equ #$02
PLATFORMCOLOR   equ #$04
PLAYFIELDSZ  equ #136
ENEMYHEIGHT  equ #17
SPRITEINIT   equ #PLAYFIELDSZ+ENEMYHEIGHT+1

;
; Initialize system (Andrew Davie's 8 byte init)
;

sysinit
    ldx #0
    txa
sysinit_clear
    dex

    txs
    pha
    bne sysinit_clear
    jsr non_system_init
    lda #0
    sta mode
    jmp main_loop
sysinit_end

; 
; Non-system initialization
;

non_system_init

    lda #0
    ldx #$7c
clear_loop
    sta #$80,x
    dex
    cpx #2
    bne clear_loop

    lda #52
    sta audio2type

    lda #$FF
    sta m0ys
    sta m1ys
    sta m2ys
    sta m3ys

    lda #<sprite1Frame1
    sta p1dataptrlo
    lda #>sprite1Frame1
    sta p1dataptrhi

    lda #0
    sta vxoffsetlo
    lda #$4d
    sta p0x
    lda #77
    sta p0vxlo

    rts
non_system_init_end

;
; main_loop: 
; 
; This is comprised of the following sections:
;     1) vertical sync/blank
;     2) kernel
;     3) overscan

main_loop

;
; Start the vertical blank interrupt
; TIME: ~4 scanlines
; ENDS: 8
; 

vblank_start
    lda #2              ; wait for horizontal sync
    sta  WSYNC          ; ""
    sta  VSYNC          ; turn on vsync
    sta  WSYNC          ; wait three lines
    sta  WSYNC          ; ""
    lda #0              ; ""
    sta  WSYNC          ; ""
    sta  VSYNC          ; turn off vsync
    lda  #48            ; Set the timer for 2812 cycles
    sta  TIM64T         ; ""
vblank_start_end

;
; If the game has been reset then the mode will be $FF
;

do_handle_reset
    lda mode
    beq do_handle_reset_end ; in init mode (0)
    cmp #128    
    beq do_handle_reset_end ; game in progress (128)
;    cmp #0              ; in init mode
;    beq do_handle_reset_end
    cmp #255
    beq do_reset
    cmp #127
    beq do_game_over
    cmp #254
    beq do_game_over
    dec mode
    jmp do_handle_reset_end
do_reset
    jsr non_system_init
    lda #253
    sta mode
    lda #0
    sta score
    sta score+1
    jmp do_handle_reset_end
do_game_over
    lda #126
    sta mode
do_handle_reset_end

    inc frameCount      ; 5, 13

;
;
;

do_audio0 ; 35
    lda shotcount       ; 3
    sta AUDV0           ; 3
;    cmp #0              ; 2
    beq do_audio0_end ; 2/3
    dec shotcount       ; 5
    lda #15             ; 2
    sec                 ; 2
    sbc shotcount       ; 2
    clc                 ; 2
    adc #24             ; 2 tweak to alter timbre of shot
    sta AUDF0           ; 3
    lda #8              ; 2 ; tweak to alter shot distortion
    sta AUDC0           ; 3
do_audio0_end

do_audio1

    lda audio2count         ; set the volume
    and #%00111111
    lsr 
    sta AUDV1

;    cmp #0                  ; if there's nothing else to play, then 
    beq do_soundtrack       ; play the soundtrack

 ;   sta AUDV1

    cmp #1
    bne keeptype
    lda #52
    sta audio2type
keeptype

    dec audio2count

    lda frameCount
    and #%00100110
    clc
    adc audio2type ; 52 or 128 or 192

    sta AUDF1

    lda audio2type
    cmp #192
    beq doRandomAudc
    cmp #132
    beq do8Audc
    lda #15
    jmp noRandomAudc
doRandomAudc
    lda frameCount
    jmp noRandomAudc
do8Audc
    lda #8
noRandomAudc
    sta AUDC1
    jmp do_audio1_end

do_soundtrack
    lda mode
    cmp #128
    bcc do_audio1_end
    lda frameCount
    and #%00000100
    bne do_audio1_end
    lda #8
    sta AUDV1
    clc
    lda frameCount
    eor random
    and #%11110111
    adc #228
    sta AUDF1
    lda #15
    sta AUDC1
do_audio1_end

adjust_missiles
    lda m0ys
    cmp #$FF
    beq adjust_1
    cmp #PLAYFIELDSZ
    bcs reset_missile_0
    clc
    adc #MISSILESPEED
    sta m0ys
    jmp adjust_1
reset_missile_0
    lda #$FF
    sta m0ys
adjust_1
    lda m1ys
    cmp #$FF
    beq adjust_2
    cmp #PLAYFIELDSZ
    bcs reset_missile_1
    clc
    adc #MISSILESPEED
    sta m1ys
    jmp adjust_2
reset_missile_1
    lda #$FF
    sta m1ys
adjust_2
    lda m2ys
    cmp #$FF
    beq adjust_3
    cmp #PLAYFIELDSZ
    bcs reset_missile_2
    clc
    adc #MISSILESPEED
    sta m2ys
    jmp adjust_3
reset_missile_2
    lda #$FF
    sta m2ys
adjust_3
    lda m3ys
    cmp #$FF
    beq adjust_missiles_end
    cmp #PLAYFIELDSZ
    bcs reset_missile_3
    clc
    adc #MISSILESPEED
    sta m3ys
    jmp adjust_missiles_end
reset_missile_3
    lda #$FF
    sta m3ys
adjust_missiles_end

    sta HMCLR           ; clear the ball init

move_missiles
    lda frameCount
    and #%0000001
;    cmp #0
    beq move_missiles_even
    jmp move_missiles_odd

move_missiles_even
    lda m0ys
    cmp #$FF
    beq move_missiles_even_1
    lda m0vxlo
;    sta temp2lo
    jsr convert_virtual_xpos
    clc
    adc #P0CENTER       ; add to align with center of ship
#if MISSILE_FOLLOWS_SHIP = 1
    lda p0x             ; missle follows ship
    clc                 ; ""
    adc #4              ; ""
#endif
    ldx #3
    jsr do_sprite_move  
move_missiles_even_1
    lda m1ys
    cmp #$FF
    beq move_missiles_even_end
    lda m1vxlo
;    sta temp2lo
    jsr convert_virtual_xpos
    clc
    adc #P0CENTER       ; add to align with center of ship
#if MISSILE_FOLLOWS_SHIP = 1
    lda p0x             ; missle follows ship
    clc                 ; ""
    adc #4              ; ""
#endif
    ldx #2
    sta HMCLR
    jsr do_sprite_move
move_missiles_even_end
    jmp move_missiles_odd_end

move_missiles_odd
    lda m2ys
    cmp #$FF
    beq move_missiles_odd_3
    lda m2vxlo
;    sta temp2lo
    jsr convert_virtual_xpos
    clc
    adc #P0CENTER       ; add to align with center of ship
#if MISSILE_FOLLOWS_SHIP = 1
    lda p0x             ; missle follows ship
    clc                 ; ""
    adc #4              ; ""
#endif
    ldx #3
    jsr do_sprite_move
move_missiles_odd_3
    lda m3ys
    cmp #$FF
    beq move_missiles_odd_end
    lda m3vxlo
;    sta temp2lo
    jsr convert_virtual_xpos
    clc
    adc #P0CENTER       ; add to align with center of ship
#if MISSILE_FOLLOWS_SHIP = 1
    lda p0x             ; missle follows ship
    clc                 ; ""
    adc #4              ; ""
#endif
    ldx #2
    sta HMCLR
    jsr do_sprite_move
move_missiles_odd_end

move_missiles_end

;
; We are keeping state for 4 missiles. Every other frame we 
; map even or odd missiles to M0/M1.
;

swap_missiles1
    lda frameCount
    and #%0000001
    beq swap_missiles1_end
    ldx m0ys
    lda m2ys
    sta m0ys
    stx m2ys
    ldx m1ys
    lda m3ys
    sta m1ys
    stx m3ys
swap_missiles1_end

    nop                 ; waste time before the HMCLR
    sta HMCLR
;
; Check the bottom debris moved and see if it needs to be
; removed
;

remove_stale_debris
    ldx debrisCount
    cpx #0
    beq remove_stale_debris_end
    dex
    lda debrisy,x
    cmp #4
    bcs remove_stale_debris_end
    dec debrisCount    

    lda debrisinfo,x
    and #%00111111
;    cmp #%00000000
    bne check_for_absorber
    lda #23
    sta audio2count
    lda #132
    sta audio2type
    lda radlevel
    lsr
    clc
    adc #128
    sta radlevel

    cmp #$FF
    bne remove_stale_debris_end
    lda #127
    sta mode
    lda #0
    sta debrisCount
    jmp remove_stale_debris_end

check_for_absorber

    lda debrisinfo,x
    and #%00111111
    cmp #%00111000
    bne remove_stale_debris_end
    lda #24
    sta audio2count
    lda #192
    sta audio2type
    lda radlevel
    asl
    sta radlevel


remove_stale_debris_end

;
; Move the debris downward and adjust horizontal screen positions 
; according to the the current scroll position. Load the visible 
; sprites into spritelist.
;

move_debris
    lda #0
    sta spritecount
    ldx debrisCount
    cpx #0 
    bne no_move_debris_end
    jmp move_debris_end
no_move_debris_end

    lda wave            ; move the debris down according to the current 
    asl
    and #%00011100      ; ""
    sta temp3           ; ""
    lda frameCount      ; ""
    and #%00000011      ; ""
    ora temp3           ; ""
    tay
    lda debrisMoveTable,y
    sta nexty

    ldx #0
move_debris_loop

    lda debrisinfo,x    ; 4
    lsr                 ; 2
    lsr                 ; 2
    lsr                 ; 2
    tay                 ; 2
    lda frameCount      ; 3
    and andTable,y      ; 5
;    cmp #0              ; 2
    bne nohorizmove     ; 2
    clc                 ; 2
    lda debrisvx,x       ; 4
    adc moveTable,y     ; 5
    sta debrisvx,x       ; 4, 43
nohorizmove

    lda debrisy,x       ; ""
    sec                 ; ""
    sbc nexty
    sta debrisy,x       ; ""

    lda debrisinfo,x
    and #%00000111        ; check to see if this debris has been removed
    cmp #%00000111
    bne continue_with_convert
    lda #$FF
    sta debrisx,x
    jmp set_to_ff
continue_with_convert
    lda debrisvx,x
    jsr convert_virtual_xpos
    sta debrisx,x          ; ""
set_to_ff
    cmp #$FF            ; if the x position is FF do not add to the list
    beq move_debris_skip ; ""

    stx temp3         ; store the original value of x

    lda debrisinfo,x ; increment the condemned counter if applicable.
    and #%00000111
;    cmp #0
    beq skipincrement
    sta temp1lo
    inc temp1lo
    lda debrisinfo,x 
    and #%11111000
    ora temp1lo
    sta debrisinfo,x

    lda #<explosionFrameTable
    sta temp2lo
    lda #>explosionFrameTable
    sta temp2hi
    jmp skipspriteselect
skipincrement

    lda debrisinfo,x  ; load the correct sprite
    and #%00111000
    lsr
    lsr
    tax
    lda spriteTable,x
    sta temp2lo
    lda spriteTable+1,x
    sta temp2hi
skipspriteselect

    lda frameCount      ; 3     ; use framecount to adjust the sprite frame
    and #%00011100      ; 2
    lsr                 ; 2

    tay
    lda (temp2lo),y
    sta temp1lo
    iny
    lda (temp2lo),y
    sta temp1hi

    ldx temp3
    txa
    ldx spritecount     ; add this debris to the list of sprites to draw
    sta spritelist,x    ; ""

    lda temp1lo
    sta sprite1lo,x
    lda temp1hi
    sta sprite1lo+8,x

    ldx temp3
    inc spritecount

move_debris_skip

    inx
    cpx debrisCount
    beq move_debris_end
    jmp move_debris_loop
move_debris_end    

;
; load the score pointers to be used later in the kernel
;

loadScore
    lda score           ; load half the player score
    and #$0F            ; and out the low nibble
    tax
    lda scoreTable,x    ;
    sta spritelist      ;
    lda scoreTable+10,x ;
    sta spritelist+1    ;

    lda score           ; load half the player score
    lsr                 ; extract the high nibble
    lsr
    lsr
    lsr
    tax
    lda scoreTable,x    ;
    sta spritelist+2      ;
    lda scoreTable+10,x  ;
    sta spritelist+3    ;

    lda score+1           ; load half the player score
    and #$0F            ; and out the low nibble
    tax
    lda scoreTable,x    ;
    sta spritelist+4      ;
    lda scoreTable+10,x  ;
    sta spritelist+5    ;

    lda score+1           ; load half the player score
    lsr                 ; extract the high nibble
    lsr
    lsr
    lsr
    tax
    lda scoreTable,x    ;
    sta spritelist+6      ;
    lda scoreTable+10,x  ;
    sta spritelist+7    ;

;
; Map the player onto the radar
;

map_player

    ldy #$FF
    lda p0vxlo          ; calculate the x coord for the map
    sec                 ; "" divide by 65
DivideBy64              ; +1
    iny                 ; 2
    sbc #64             ; 2
    bcs DivideBy64      ; 2 repeat up to 3 times, result will be in [0,3]

    lda p0vxlo          ; divide by 8 and lookup bit offset
    lsr
    lsr
    lsr
    tax
    lda map_debris_table,x
    sta radarRam,y
map_player_end

;
; Initialize the first sprite position, this triggers finding 
; the next sprite in the kernel.
; TIME: 5 cycles
; 

init_first_sprite
    lda #SPRITEINIT     ; 2
    sta nexty           ; 3
init_first_sprite_end

;
; set up the ball, we use it to hide HMOVE bars
; Time: does a WSYNC and then 122 cycles.
;

init_ball
    sta WSYNC           ; 3, 0
	STA	RESBL	        ; 3, 3  ; Reset the ball
	LDA	#$22	        ; 2, 5  ; Set the horizontal ball motion (+2)
	STA	HMBL            ; 3, 8  ; 
	STA	ENABL	        ; 3, 11 ;Enable the ball (02)
    lda #%00110100      ; 2, 13  
    sta CTRLPF          ; 3, 16
init_ball_end

;
; End the vertical blank
;

vblank_end
waitOnVblank
    lda INTIM
    bne waitOnVblank
    lda #0              ; turn off the vblank
    sta VBLANK          ; ""
    sta WSYNC           ; wait for the next line.
    sta HMOVE           ; this HMOVE is paired with the init_ball routine above
vblank_end_end

;
; Draw the score
;
    lda #SKYCOLOR       ; 2, 4
    sta COLUBK          ; 3, 7
draw_score
    sta WSYNC           ; 3, -
    lda #1              ; 2, 2
    sta HMCLR           ; 3, 5
    sta NUSIZ0          ; 3, 8
    sta NUSIZ1          ; 3, 11
    lda #SCORECOLOR     ; 2, 13
    sta COLUP0          ; 3, 16
    sta COLUP1          ; 3, 19
    SLEEP 20            ; 20, 39
    sta RESP0           ; 3, 42
    sta RESP1           ; 3, 45
    lda #%11110000      ; 2, 47
    sta HMP0            ; 3, 50
    lda #%00000000      ; 2, 52
    sta HMP1            ; 3, 55
    sty nexty           ; 3, 58
    ldy #7              ; 2, 60
    sta WSYNC           ; 3, 3
    sta HMOVE           ; 3, 3
    nop                 ; 2, 5
score_loop              ; +1, 5
    dey                 ; 2, 7
    lda (spritelist+6),y  ; 6, 13
    sta GRP0            ; 3, 16
    lda (spritelist+4),y ; 6, 22
    sta GRP1            ; 3, 25
    lda (spritelist),y ; 6, 31
    tax                 ; 2, 33
    lda (spritelist+2),y ; 6, 39
    sleep 8             ; 8, 47
    sta GRP0            ; 3, 50 
    stx GRP1            ; 3, 53
    sta WSYNC           ; 3, 66
    cpy #0              ; 2, 2
    bne score_loop      ; 2, 4
    lda #0              ; 2, 6
    sta GRP1            ; 3, 9
    sta GRP0            ; 3, 12
    lda  #%00010000     ; 2, 14

;
; Load smartbomb graphics. 
; TIME: 23
; ENDS: 37
;

load_smart_bombs
    lda bombState       ; 3, 3
    and #%00000011      ; 2, 5
    tax                 ; 2, 7
    lda leftSmartBombTable,x    ; 5, 12 
    sta spritelist+2            ; 3, 15
    lda leftSmartBombTable+4,x    ; 5, 12 
    sta spritelist+3            ; 3, 15
    lda rightSmartBombTable,x   ; 5, 20
    sta spritelist+4            ; 3, 23
    lda rightSmartBombTable+4,x   ; 5, 20
    sta spritelist+5            ; 3, 23
load_smart_bombs_end 

;
; Draw the smart bombs. 
; 

draw_smart_bombs
    ldy #3              ; 2, 2
    lda frameCount      ; 3, 5
    sta COLUP0          ; 3, 8
    sta COLUP1          ; 3, 11
    sta WSYNC
    sta WSYNC           ; 3, 14 ; there are about 40 cycles to spare here
    sleep 5
score_loop2             ; +1, 5
    dey                 ; 2, 7
    lda #0              ; 2, 9
    sleep 3             ; 3, 12
    sta GRP0            ; 3, 15
    lda (spritelist+2),y ; 6, 21
    nop                 ; 2, 23
    sta GRP1            ; 3, 26
    ldx #0              ; 2, 28
    sleep 5             ; 5, 33
    lda (spritelist+4),y ; 6, 39
    sleep 7             ; 7, 46
    sta GRP0            ; 3, 49 
    stx GRP1            ; 3, 52
    sta WSYNC           ; 3, 55
    cpy #0              ; 2, 2
    bne score_loop2     ; 2, 4
    lda #0              ; 2, 6
    sta GRP1            ; 3, 9
    sta GRP0            ; 3, 12
    lda #$0E            ; 2, 14
    sta COLUP0          ; 3, 17
    sta COLUP1          ; 3, 20
draw_smart_bombs_end


;
; Draw the sky area between the score and the mountains. During this time we
; also need to load the spritelist and then preload the first debris sprite. 
; This ends up taking a variable amount of time so we'll just set a timer 
; here and do the things we need then wait for it to expire. 
;

    lda #7              ; 2, 22 ;
    sta TIM64T          ; 3, 25 ;

;
; load the list of currently visible debris sprites 
; into spriteList.
; TIME: 184 (max)
;

load_visible_sprites
    ldx #$FF            ; 2, 7       ; index
    ldy #0              ; 2, 9       ; for spritecount
    jmp lvs_skip        ; 3
lvs_loop                ; max loop time (when debriscount = 8) is 175 cycles
    lda debrisx,x       ; 4         ; if the x position is FF (off screen), then skip
    cmp #$FF            ; 2 
    beq lvs_skip        ; 2/3
    stx spritelist,y    ; 4
    iny                 ; 2
lvs_skip                ; + 1
    inx                 ; 2
    cpx debrisCount     ; 3         
    bne lvs_loop        ; 3            ; LOOP TIME = 22 * 8 - 1 = 175
    sty spritecount     ; 3,  184      ; set spritecount
load_visible_sprites_end

;
; preload pointers for the first debris sprite to be used later 
; in the kernel.
; 
; MAX TIME : 
;

preload_sprite
    lda #$EF            ; 2, 2  ; Yes, this is $EF
    sta nexty           ; 3, 5
    lda spritecount     ; 3, 8
    beq preload_sprite_end ; 2, 10

    ldx #0              ; 2, 12
    lda sprite1lo,x     ; 4, 16
    sta p1dataptrlo     ; 3, 19
    lda sprite1lo+8,x   ; 4, 23
    sta p1dataptrhi     ; 3, 26

    ldy #17             ; 2, 28 ;
    lda (p1dataptrlo),y ; 6, 34 ;
    sta p1colorlo       ; 3, 37 ;
    iny                 ; 2, 39 ;
    lda (p1dataptrlo),y ; 6, 45 ;
    sta p1colorhi       ; 3, 48 ;

    ldy #0              ; 2, 50
    ldx spritelist,y    ; 4, 54
    lda debrisy,x       ; 4, 58
    sta nexty           ; 3, 61
    lda debrisx,x       ; 4, 65
    ldx #1              ; 2, 67
    jsr do_sprite_move  ; ~91, 158
preload_sprite_end

sky_wait
    lda INTIM           ; 3, --
    bne sky_wait        ; 2, --


    lda  #%00010000     ; 2, 14
    sta  NUSIZ0         ; 3, 17
    sta  NUSIZ1         ; 3, 20

    sta WSYNC

;
; This is where we set the background color for the fade-in at the start 
; of the game and various other visual effects. This must fit in a 
; single scan line
; 

    lda #$0c            ; 2, 2
    sta COLUBK          ; 3, 5
    lda audio2type      ; 3, 8
    cmp #52             ; 2, 10   
    beq doFadeIn        ; 2, 12
    cmp #132            ; 2, 14
    beq setYellowEffect ; 2, 16
    cmp #128            ; 2, 18
    beq setRedEffect    ; 2, 20
    lda #$80            ; 2, 22
    jmp doneSetEffectColor ; 3, 25
setYellowEffect         ; +1 17
    lda #$10            ; 2, 19
    jmp doneSetEffectColor  ; 3, 22
setRedEffect            ; +1 21
    lda #$30            ; 2, 23
doneSetEffectColor      ; +1 26
    sta temp2hi         ; 3, 29
    lda audio2count     ; 3, 32  
;    cmp #0              ; 2, 34
    beq doFadeIn        ; 2, 36
    lsr                 ; 2, 38
    lsr                 ; 2, 40
;    lsr
;    asl
    cmp #0              ; 2, 42
    beq setBackground   ; 2, 44
    clc                 ; 2, 46
    adc temp2hi         ; 3, 49
    jmp setBackground   ; 3, 52
doFadeIn                ; +1 [13,37]
    lda mode            ; 3, 40
    cmp #128            ; 2, 42  ; if mode < 128 we set the background color to 0
    bcc setZero         ; 2, 44
    sec                 ; 2, 46
    sbc #128            ; 2, 48
    asl                 ; 2, 50
    lsr                 ; 2, 52
    lsr                 ; 2, 54
    lsr                 ; 2, 56
    lsr                 ; 2, 58
    jmp setBackground   ; 3, 61
setZero                 ; +1, 45
    lda #0              ; 2, 47
setBackground           ; +1, [28, 61]
    sta WSYNC           ; 
    sta COLUBK          ; 3, 3

;    jmp draw_playfield
;AdjustCodeOrg1
;    .byte #0

draw_playfield
    lda #1              ; 2, 5
    sta temp3           ; 3, 8
    ldx #PLAYFIELDSZ    ; 2, 10
do_playfield            ; +1, [9,23]
    sec                 ; 2, 25 ; ---------------------------------------------
    txa                 ; 2, 27 ;
    sbc m1ys            ; 3, 30 ; draw missile 0, max 16 cycles
    adc #MISSILESIZE    ; 2, 32 ; 
    lda	#2		        ; 2, 34 ; 
    adc #$ff            ; 2, 36 ;
    sta ENAM0           ; 3, 39 ; ---------------------------------------------
    sec                 ; 2, 41
    txa                 ; 2, 43 ; ---------------------------------------------
    sbc m0ys            ; 3, 46 ; draw missile 1, max 16 cycles
    adc #MISSILESIZE    ; 2, 48 ; 
    lda	#2		        ; 2, 50 ; 
    adc #$ff            ; 2, 52 ;
    sta ENAM1           ; 3, 55 ; ---------------------------------------------
    txa                 ; 2, 57 ; ---------------------------------------------
    sec                 ; 2, 59 ; set up player1 - max  16 cycles
    sbc nexty           ; 3, 62 ; 
    adc #ENEMYHEIGHT    ; 2, 64 ;
    bcc finishedSprite  ; 2, 66 ;
    tay                 ; 2, 68 ; 4 FREE CYCLES !!
    sta WSYNC           ; 3, 71 ; ---------------------------------------------
    lda (p1dataptrlo),y ; 6, 6  ; ---------------------------------------------
    sta GRP1            ; 3, 9  ; draw player 1 w/ colors - max 17 cyles
    lda (p1colorlo),y   ; 5, 14 ;
    sta COLUP1          ; 3, 17 ; ---------------------------------------------
    dex                 ; 2, 19 ;    
    bne do_playfield    ; 2, 21 ;
    jmp draw_playfield_end ; 3, 24 ; 
finishedSprite          ; +1, 67 ;
    cmp #$FF            ; 2, 69 ; we just finished the last line of a sprite
    beq load_next_sprite; 2, 71 ; 
    sta WSYNC           ; 3, 74 ; 2 FREE CYCLES
    dex                 ; 2, 2  ;  
    bne do_playfield    ; 2, 4  ;
    jmp draw_playfield_end ; 3, 7  ;
load_next_sprite        ; +1, 72
    sta WSYNC           ; 3, 75 ; 1 FREE CYCLE

    lda temp3           ; 3, 3  ; 
    cmp spritecount     ; 3, 6  ;
    bcs no_more_sprites ; 2, 8 ;

    stx temp1lo         ; 3, 11 ; dynamically load sprite data 
    tax                 ; 2, 13
    lda sprite1lo,x     ; 4, 17 ; ""
    sta p1dataptrlo     ; 3, 20 ; ""
    lda sprite1lo+8,x   ; 4, 24 ; ""
    sta p1dataptrhi     ; 3, 27 ; ""

    ldy #17             ; 2, 29 ;
    lda (p1dataptrlo),y ; 6*, 35 ; 
    sta p1colorlo       ; 3, 38 ;
    iny                 ; 2, 40 ;
    lda (p1dataptrlo),y ; 6*, 46 ;
    sta p1colorhi       ; 3, 49 ;

    ldy temp3           ; 3, 52 ; get the next sprite from spritelist
    ldx spritelist,y    ; 4, 56 ; ""
    inc temp3           ; 3, 59 ; ""
    lda debrisy,x        ; 4, 63 ; ""
    sta nexty           ; 3, 66 ; ""

    lda debrisx,x        ; 4, 70 ; setup A and X for do_sprite_move

    sta WSYNC
    sec                 ; 2, 2
DivideLoop1             ; This loop MAX 54 cycles if A is < 160, MIN 4
    sbc #15             ; 2, [4
    bcs DivideLoop1     ; 2/3 [6,56]        
    tay                 ; 2, [8,58]
    lda FineAdjustTableEnd,Y    ; 5, [13,63]
    ldx #1
    sta HMP0,X          ; 3  [19,69]
    sta RESP0,x         ; 4  [23,73]
    sta WSYNC           ; 3  [26,76]
    sta HMOVE           ; 3, 3

    ldx temp1lo         ; 3, 12 ;
    dex                 ; 2, 14 ;
    dex                 ; 2, 16 ;
    dex                 ; 2, 18 ;
    jmp do_playfield    ; 2, 20 ; checking dex is not required here - we know there is at least one more sprite
no_more_sprites         ; +1, 9 ;
    lda #$ef            ; 2, 11
    sta nexty           ; 3, 14
    dex                 ; 2, 16 ;
    beq draw_playfield_end ; 2, 18 ;
    jmp do_playfield    ; 2, 20 ;
draw_playfield_end      ; [7,24]

;
; Lunar hills just above the Lunokhod. If there is a debris sprite that 
; overlaps into this area it will be drawn. 
; 

draw_hills
    sta HMCLR           ; 3, 27 ; so HMOVE's will not reposition debris
    lda scrollpos       ; 3, 30 
    and #%11111100      ; 2, 32
    clc                 ; 2, 34
    adc #4              ; 2, 36 
    tax                 ; 2, 38 
    lda #SNOWCOLOR      ; 2, 40 
    sta COLUPF          ; 3, 43 
    lda #$FF            ; 2, 45
    sta temp1lo         ; 3, 48 ; start the counter at FF (-1) this is 
                                ; allows us to continue using skipdraw with 
                                ; the current "nexty" for debris
hills_loop              ; +1, 48

    sec                 ; 2, 50 ; skipdraw
    sbc nexty           ; 3, 53 ; 
    adc #ENEMYHEIGHT    ; 2, 55 ;
    bcs doDrawSprite    ; 2, 57 ;
    sta WSYNC           ;       ; no sprite
    sta HMOVE           ; 3, 3  ;
;    lda #0              ; 2, 5
;    sta GRP1            ; 3, 8
    lda PFData0-1,X     ; 4, 7
    sta PF0             ; 3, 10
    jmp overSprite      ; 4, 14
doDrawSprite            ; 1, 58 ; draw the sprite
    tay                 ; 2, 60
    sta WSYNC           ; 3, 63
    sta HMOVE           ; 3, 3
    lda PFData0-1,X     ; 4, 7
    sta PF0             ; 3, 10
    lda (p1dataptrlo),y ; 6, 16
    sta GRP1            ; 3, 19

overSprite              ; [14,19]
    lda PFData1-1,X     ; 4, 23
    sta PF1             ; 3, 26
    lda PFData2-1,X     ; 4, 30
    sta PF2             ; 3, 33

    dex                 ; 2, 35
    dec temp1lo         ; 5, 40
    lda temp1lo         ; 3, 43
    cmp #$FB            ; 2, 45
    bne hills_loop      ; 2, 47


draw_hills_end

;
; Draw the sled and rail
;

draw_sled_and_rail

    lda #0              ; 2, 49 
    sta GRP1            ; 3, 52  ; turn off player1 graphics 
    sta ENAM0           ; 3, 55  ; turn off missiles
    sta ENAM1           ; 3, 58  ; 

    sta WSYNC
    sta HMOVE           ; 3, 3 ; 

    sta COLUPF          ; 3, 6  ; set pf color to black for HMOVE hiding
    sta PF0             ; 3, 9  ; clear playfield data
    sta PF1             ; 3, 12 ; 
    sta PF2             ; 3, 15 ;

    lda #SNOWCOLOR      ; 2, 17 ; change the background color to white
    sta COLUBK          ; 3, 20 ; ""

    lda p0x             ; 3, 23
    ldx #0              ; 2, 25
    jsr do_sprite_move  ; -, 
    ldx #10
do_sled
    lda PlayerSprite,X
    sta GRP0
    lda PlayerColor,X
    sta COLUP0
    sta WSYNC
    dex
    cpx #0
    bne do_sled

    lda #RAILCOLOR
    sta COLUBK
    lda PlayerSprite,X
    sta GRP0
    sta WSYNC

    lda #PLATFORMCOLOR
    sta COLUBK
    lda #0
    sta GRP0

    ldy #172

draw_sled_and_rail_end

draw_gauge
    lda #%00000111      ; 2
    sta NUSIZ0          ; 3
    lda #63             ; 2
    ldx #0              ; 2
    jsr do_sprite_move  ; -, 9
    lda radlevel       ; 3, 12
    sta GRP0            ; 3, 15

    lda frameCount
    and #%00000111
    adc #$14

;    lda #GAUGECOLOR     ; 2, 17
    sta COLUP0          ; 3, 20
    sta WSYNC
    sta HMCLR
    sta WSYNC
    sta HMOVE
    lda #0
    sta GRP0
    lda #%11110000      ; 2
    sta PF2             ; 3
    lda #%00110001      ; 2
    sta CTRLPF          ; 3
    lda #PLATFORMCOLOR
    sta COLUPF
draw_gauge_end

;
;Draw the radar and surrounding area.
;

    jmp draw_radar
AdjustCodeOrg2
    .byte #0
    .byte #0
    .byte #0
    .byte #0
    .byte #0
    .byte #0

draw_radar
    sta WSYNC
    sta HMOVE
    lda #1              ; 2, 2
    sta HMCLR           ; 3, 5
    sta NUSIZ0          ; 3, 8
    sta NUSIZ1          ; 3, 11

    lda radarColor      ; 3, 14
    asl                 ; 2, 16
    sta radarColor      ; 3, 19
    and #%10000000      ; 2, 21
    beq normalRadarColor; 2, 23
    lda #$1e             ; 2, 25
    jmp setColor        ; 3, 28
normalRadarColor        ; +1, 24
    lda #RADARCOLOR     ; 2, 26
    nop                 ; 2, 28
setColor
    sta COLUP0          ; 3, 31
    sta COLUP1          ; 3, 34
    nop                 ; 2, 36

;    SLEEP 17            ; 20, 39
    sta RESP0           ; 3, 42
    sta RESP1           ; 3, 45
    lda #%11110000      ; 2, 47
    sta HMP0            ; 3, 50
    lda #%00000000      ; 2, 52
    sta HMP1            ; 3, 55
;    sty nexty           ; 3, 58
    ldx #32             ; 2, 60
    lda #0
    sta COLUPF          ; 3, 6
    sta WSYNC           ; 3, 0
    sta HMOVE           ; 3, 3
    nop                 ; 2, 5
radar_loop              ; +1, 5
    dex                 ; 2, 7
    dex                 ; 2, 9
    dex                 ; 2  11
    dex                 ; 2, 13
    lda radarRam,x      ; 4, 17
    sta GRP0            ; 3, 20
    lda radarRam+1,x    ; 4, 24
    sta GRP1            ; 3, 27
    ldy radarRam+3,x    ; 4, 31
    lda radarRam+2,x    ; 4, 35
    sleep 9             ; 9, 44
    sta GRP0            ; 3, 47 
    sty GRP1            ; 3, 50

    lda radarColor      ; 3, 53
    asl                 ; 2, 55
    sta radarColor      ; 3, 58

    and #%10000000      ; 2, 60
    beq normalRadarColor2 ; 2, 62
    lda #$1e            ; 2, 64
    jmp setColor2       ; 3, 67
normalRadarColor2       ; +1, 63
    lda #RADARCOLOR     ; 2, 65
    nop                 ; 2, 67
setColor2
    sta COLUP0          ; 3, 70
    sta COLUP1          ; 3, 73

    sta WSYNC           ; 3, 75
    cpx #4              ; 2, 2
    bne radar_loop      ; 2, 4

    lda #RADARCOLOR2    ; 2, 6
    sta COLUP1          ; 3, 9
    sta COLUP0          ; 3, 12
    lda radarRam        ; 3, 15
    nop                 ; 2, 17
    sta GRP0            ; 3, 20 ; this must be on cycle 19 or 20
    lda radarRam+1      ; 3, 23 
    sta GRP1            ; 3, 26 ; this must be on cycle 26 or 27
    ldy radarRam+3      ; 3, 29
    lda radarRam+2      ; 3, 32
    sleep 12            ; 12, 44
    sta GRP0            ; 3, 47 ; these three must start on cycle 47
    sty GRP1            ; 3, 50
    sta WSYNC

    sleep 10            ; 10, 10
    ldx #0              ; 2, 12
    lda radarRam,x      ; 4, 16
    sta GRP0            ; 3, 19 ; this must be on cycle 19 or 20
    lda radarRam+1,x    ; 4, 23 
    sta GRP1            ; 3, 26 ; this must be on cycle 26 or 27
    ldy radarRam+3,x    ; 4, 30
    lda radarRam+2,x    ; 4, 34
;    stx temp1lo         ; 3, 37
;    ldx #0              ; 2, 39
    sleep 10             ; 10, 44
    sta GRP0            ; 3, 47 ; these three must start on cycle 47
    sty GRP1            ; 3, 50
;    stx GRP0            ; 3, 53
;    stx GRP1            ; 3, 56

    sta WSYNC
    lda #$0e     ; 2, 16
    sta COLUP0          ; 3, 19
    sta COLUP1          ; 3, 14
    lda #0              ; 2, 6
    sta GRP1            ; 3, 9
    sta GRP0            ; 3, 12
    sta PF2
    sta NUSIZ0         ; 3, 26
    sta NUSIZ1         ; 3, 29
    sta WSYNC
    sta WSYNC
    
overscan_start
    lda #2              ; turn on the vblank
    sta VBLANK          ; ""
    lda  #32          ; wait for 4544 cycles then sync
    sta  TIM64T         ; ""
overscan_start_end

;
; Process collisions
; 

do_collisions
    lda CXM0P           ; 3     ; Check if any collisions were measured with missile 0
    and #%10000000      ; 2     ; ""
;    cmp #0              ; 2     ; ""
    beq no_m0_collision ; 2     ; ""

    ldx #0              ; 2     ; Walk through the debris list checking for collisions
m0_collision_loop       ; +1    ; ""
    lda debrisy,x        ; 4     ; ""
    sec                 ; 2     ; ""
    sbc m1ys            ; 3     ; ""
    cmp #$FD            ; 2     ; ""
    bcs found_m0        ; 2     ; ""
    cmp #ENEMYHEIGHT    ; 2     ;
    bcc found_m0        ; 2     ; ""
    jmp no_pm0_collision ; 3    ; ""
found_m0
    lda debrisinfo,x  ; if the debris is in condemned state this is not a collision
    and #%00000111      ; ""
;    cmp #0              ; ""
    bne no_pm0_collision ; ""
    jsr remove_debris
    lda #$FF
    sta m1ys
    jmp no_m0_collision
no_pm0_collision
    inx
    cpx debrisCount
    bcc m0_collision_loop

no_m0_collision
    lda CXM1P           ; 3     ; Check if missile 1 has a collision
    and #%01000000      ;       ; ""
;    cmp #0              ;       ; ""
    beq no_m1_collision ;       ; ""

    ldx #0
m1_collision_loop
    lda debrisy,x
    sec 
    sbc m0ys
    cmp #$FD
    bcs found_m1
    cmp #ENEMYHEIGHT
    bcc found_m1
    jmp no_pm1_collision
found_m1
    lda debrisinfo,x  ; if the debris is in a condemned state this is not a collision
    and #%00000111      ; ""
;    cmp #0              ; ""
    bne no_pm1_collision ; ""
    jsr remove_debris
    lda #$FF
    sta m0ys
    jmp no_m1_collision
no_pm1_collision
    inx
    cpx debrisCount
    bcc m1_collision_loop

no_m1_collision
    sta CXCLR
do_collisions_end

swap_missiles2
    lda frameCount      ; every other frame we swap the missiles
    and #%0000001
;    cmp #0
    beq swap_missiles2_end

    ldx m0ys            ; swap missiles 0 and 2
    lda m2ys
    sta m0ys
    stx m2ys

    ldx m1ys            ; swap missiles 1 and 3
    lda m3ys
    sta m1ys
    stx m3ys
swap_missiles2_end

;
; Check if the reset button was pressed
;

check_for_reset
    lda SWCHB          ; check to see if the reset or select switch was triggered
    and #%00000001
    bne check_for_reset_end ; 0 means reset is down
    lda #$FF
    sta mode
    jmp check_trig_reset
check_for_reset_end

    lda mode
;    cmp #0
    bne check_trig_reset_end
    
check_trig_reset
    lda INPT4           ; first check the trigger
    bmi check_trig_reset_end
    lda #$FF
    sta mode
    jmp overscan_end
check_trig_reset_end

    lda mode
    cmp #128
    bcs do_bombs
    jmp skip_positioning

;
; Check for down movmement (update bombs)
;

do_bombs
    lda bombState       ; 3 ; are there any bombs left?
    and #%00000011      ; 2 ; ""
    cmp #3              ; 2 ; ""
    beq do_bombs_end    ; 2 ; ""

    lda #%00100000      ; 2 ; joystick down?
    bit SWCHA           ; 3 ; ""
    bne downNotPressed  ; 2 ;
    
    lda bombState       ; 3 ; down is pressed, we'll mark the 
    ora #%10000000      ; 2 ; debounce and exit
    sta bombState       ; 3 ; 
    jmp do_bombs_end    ; 3 ; 

downNotPressed          ; +1 down is not pressed, check for debounce
    lda bombState       ; 3 ; 
    and #%10000000      ; 2 ; 
    beq do_bombs_end    ; 2 ; 

    lda bombState       ; 3 ; 
    clc                 ; 2 ; 
    adc #129            ; 2 ; add 1 and roll off the debounce bit
    sta bombState       ; 3 ; 

    lda #31             ; 2
    sta audio2count     ; 3
    lda #128            ; 2
    sta audio2type      ; 3



    ; loop through the debris, remove and score any that 
    ; are visible
    ldx #7              ; 2
dsb_loop

    lda debrisx,x
    cmp #$FF
    beq dsb_notvisible
    
    jsr remove_debris

dsb_notvisible
    dex
    bne dsb_loop
dsb_done




do_bombs_end

;
; Process joystick movement to set the horizontal vector for 
; player 0.
;

do_joystick
    lda INPT4           ; first check the trigger
    bmi no_button
    jmp create_missile
create_missile_cb
no_button
	lda #%01000000	    ; Left?
    bit SWCHA           ; 
	bne do_joystick_1   ; Nope, check Right
    dec p0s
    jmp check_bounds
do_joystick_1
	lda #%10000000	    ; Right?
    bit SWCHA           ; 
    bne do_slowdown     ; Nope, start slowing down
    inc p0s
    jmp check_bounds
do_slowdown
    lda frameCount
    and #%00000111
;    cmp #0
    bne do_joystick_end
    lda p0s
;    cmp #0              ; not moving
    beq do_joystick_end
    bmi do_left_slowdown
    dec p0s
    jmp check_bounds
do_left_slowdown
    inc p0s
check_bounds
    lda p0s
    cmp #MINSPEED
    bmi less_than_1
    cmp #MAXSPEED+1
    bpl greater_than_1
    jmp do_joystick_end
less_than_1
    lda #MINSPEED
    sta p0s
    jmp do_joystick_end
greater_than_1
    lda #MAXSPEED
    sta p0s
do_joystick_end

;
; If there was no joystick movement, then skip the positioning routines.
;
    lda p0s
    bne p0_position
    jmp skip_positioning

;
; p0_position determine the position for player 0, which also 
; determines the virtual screen offset
; 
p0_position
    lda p0s
    clc
    adc p0vxlo
    sta p0vxlo
p0_position_end

;
; Compute the screen position of the player and scroll offset
; 

calc_offset
    lda vxoffsetlo
    sta temp1lo
    lda p0s
    clc
    adc p0x
    cmp #MINXPOS
    bcc adjustOffsetDown
    cmp #MAXXPOS+1
    bcs adjustOffsetUp
    jmp noAdjustOffset
adjustOffsetDown
    sta temp3
    lda #MINXPOS
    sec
    sbc temp3
    sta temp3
    lda vxoffsetlo
    sec
    sbc temp3
    sta vxoffsetlo

    lda scrollpos
    sec
    sbc temp3
    sta scrollpos
    cmp #200
    bcs wrapUp
    jmp scrollDone
wrapUp
    clc
    adc #80
    sta scrollpos
scrollDone


    jmp noAdjustOffset
adjustOffsetUp
    sec
    sbc #MAXXPOS
    sta temp3
    lda vxoffsetlo
    clc
    adc temp3
    sta vxoffsetlo

    lda scrollpos
    clc
    adc temp3
    sta scrollpos
    cmp #80
    bcs wrapDown
    jmp scrollDone2
wrapDown
    sec
    sbc #80
    sta scrollpos
scrollDone2


noAdjustOffset

    lda p0vxlo
    jsr convert_virtual_xpos
    sta p0x

calc_offset_end


skip_positioning

create_debris
    lda mode
    cmp #128
    beq ce_continue
    jmp create_debris_end
ce_continue
    lda debrisCount      ; make sure there is a slot left
    cmp #8             ; ""
    beq create_debris_end; ""
    lda debrisy            ; make sure there is enough space for a new debris
    cmp #PLAYFIELDSZ-8          ; ""
    bcs create_debris_end; ""
    ldx #6             ; shift the list down (this can take up to 350 cycles - ouch!)
shift_down              ; +1 ""
    lda debrisvx,x    ; 4 ""
    sta debrisvx+1,x  ; 4 ""
    lda debrisinfo,x    ; 4 ""
    sta debrisinfo+1,x  ; 4 ""
    lda debrisy,x    ; 4 ""
    sta debrisy+1,x  ; 4 ""
    cpx #0              ; 2 ""
    beq done_shift_down ; 2 ""
    dex                 ; ""
    jmp shift_down      ; "
done_shift_down
    lda #PLAYFIELDSZ+11
    sta debrisy

    lda random          ; 3 select a horiz position
    asl                 ; 2 ""
    asl                 ; 2 ""
    eor p0vxlo          ; 3 ""
    adc random          ; 3 ""
    adc frameCount      ; 3 ""
    sta random          ; 3 ""
    sta debrisvx        ; 3 ""

    and #%00000111      ; set the debris info (type of sprite, fall direction)
    tax

    cmp #%00000111      ; as the wave number increases, it becomes more 
    bne doNotSetToRock  ; likely that we will reassign the absorber as a radioactive thing

    lda wave            ; load the current wave
    lsr                 ; divide by 2
    clc                 ; add 127
    adc #127            ; ""
    cmp random          ; if the wave value is greater than random, we
    bcc doNotSetToRock  ;     make the absorber a rock
    ldx #%00000101      ; 
doNotSetToRock
    txa
    asl
    asl
    asl

    sta debrisinfo       ; ""
 
    inc debrisCount
    lda debrisCount
    cmp #8
    bcc create_debris_end
    lda #8
    sta debrisCount

create_debris_end

    lda #0
    sta COLUBK          ; so the first line on the top will be black

;
; Clear the RAM that we will use to build the radar. 
; Unrolled to reduce cycle use.  (110 cycles) 
;

    lda #$00            ; 2, 11
    sta radarRam        ; 3, 14
    sta radarRam+1      ; 3, 17
    sta radarRam+2      ; 3, 20
    sta radarRam+3      ; 3, 23
    sta radarRam+4      ; 3, 26
    sta radarRam+5      ; 3, 29
    sta radarRam+6      ; 3, 32
    sta radarRam+7      ; 3, 35
    sta radarRam+8      ; 3, 38
    sta radarRam+9      ; 3, 41
    sta radarRam+10     ; 3, 44
    sta radarRam+11     ; 3, 47
    sta radarRam+12     ; 3, 50
    sta radarRam+13     ; 3, 53
    sta radarRam+14     ; 3, 56
    sta radarRam+15     ; 3, 59
    sta radarRam+16     ; 3, 62
    sta radarRam+17     ; 3, 65
    sta radarRam+18     ; 3, 68
    sta radarRam+19     ; 3, 71
    sta radarRam+20     ; 3, 74
    sta radarRam+21     ; 3, 77 ----- scanline + 1
    sta radarRam+22     ; 3, 2
    sta radarRam+23     ; 3, 5
    sta radarRam+24     ; 3, 8
    sta radarRam+25     ; 3, 11
    sta radarRam+26     ; 3, 14
    sta radarRam+27     ; 3, 17
    sta radarRam+28     ; 3, 20
    sta radarRam+29     ; 3, 23
    sta radarRam+30     ; 3, 26
    sta radarRam+31     ; 3, 29
    iny                 ; 2, 31
    iny                 ; 2, 33



;
; Draw our debris onto the radar. radar memory is a 4x8 byte area arranged
; by row:
;  
;                            column
;                 00       01       02       03 
;        00    00000000 00000000 00000000 00000000
;        01    00000000 00000000 00000000 00000000
;      r 02    00000000 00000000 00000000 00000000
;      o 03    00000000 00000000 00000000 00000000
;      w 04    00000000 00000000 00000000 00000000
;        05    00000000 00000000 00000000 00000000
;        06    00000000 00000000 00000000 00000000
;        07    00000000 00000000 00000000 00000000
;

map_debris
    lda debrisCount
;    cmp #0 
    beq map_debris_done

    ldx #0
    stx radarColor
    lda debrisy         ; calculate what will be the starting y coord on the map
    sec                 ; "" divide by 21
DivideBy22              ; +1
    inx                 ; 2
    sbc #22             ; 2
    bcs DivideBy22      ; 2 repeat up to 6 times, result will be in [0,6]
;    inx                 ; add 1 to make it [1,7]
    stx temp1hi
    ldx #0
map_debris_loop

    lda radarColor
    asl 
    sta radarColor

    lda debrisinfo,x    ; skip the enmy if it has been condemned
    and #%00000001      ; ""
    cmp #%00000001      ; ""
    beq skip_debris     ; ""

    lda debrisinfo,x
    and #%00111000
;    cmp #0
    bne notRadioactive
    inc radarColor
notRadioactive

    ldy #0
    lda debrisvx,x 
    cmp #64
    bcc no1a
    iny
no1a
    cmp #128
    bcc no2a
    iny
no2a
    cmp #192
    bcc no3a
    iny
no3a
    sty temp1lo

    lda temp1hi
    asl                 ; "" (multiple result by 4)
    asl                 ; ""

    clc                 ; Add row and column offsets together to 
    adc temp1lo          ; get our insert offset
    sta temp1lo          ; ""

    lda debrisvx,x       ; calculate the remainder put it in Y
    lsr                 ; ""
    lsr                 ; ""
    lsr                 ; ""
    tay                 ; ""

    lda map_debris_table,y
    sta temp2lo

    lda temp1lo
    tay 
    lda radarRam,y
    ora temp2lo
    sta radarRam,y    
skip_debris
    dec temp1hi
    beq map_debris_done
    inx
    cpx debrisCount
    bcc map_debris_loop
map_debris_done


overscan_end
waitOnOverscan          ; wait for overscan to finish
    lda INTIM           ; ""
    bne waitOnOverscan  ; ""
overscan_end_end

    jmp main_loop

;
; The trigger was pressed, create a missile if we can
;

create_missile

    lda shotcount
;    cmp #0
    bne no_free_missiles
    ldx #8
check_missile_n
    cpx #0
    beq no_free_missiles
    dex
    dex
    lda m0vxlo+1,x
    cmp #$FF
    bne check_missile_n
    lda #7
    sta shotcount
    lda #0
    sta m0vxlo+1,x
    lda p0vxlo          ; add the P0center to align with the ship
    sta m0vxlo,x
no_free_missiles
    jmp create_missile_cb


;
; Remove debris at position X, add a point to the score.
;     * Set the condemned countdown timer in debrisinfo (0xxx0000) to 001
;     * Set the sprite index in debrisinfo (00000XXX) to 100
;     * Result: 0001100

remove_debris
    lda audio2type
    cmp #52
    bne skipNoise
    lda #31            ; 2
    sta audio2count     ; 3
skipNoise
;    ldy #1
    lda debrisinfo,x  ; 4
;    and #%00111000      ; 2
;    cmp #%00111000
;    bne doNotAdd10
;    ldy #10
;doNotAdd10
    ora #%00000001      ; 2
    sta debrisinfo,x  ; 4

    lda #2
    sed                 ; Add 2 to the player's score
    clc                 ; ""
    adc score           ; ""
    sta score           ; ""
    lda #0              ; ""
    adc score+1         ; ""
    sta score+1         ; ""
    cld                 ; ""

    

    lda score
    bne noWaveIncrement
    lda wave
    cmp #254
    bcs noWaveIncrement
    inc wave
    inc wave 
noWaveIncrement

    rts                 ; 6
remove_debris_end


;
; Horizontal position routine. Found by members of the Stella mailing list 
; in code for Atari Battlezone. HMOVE needs to be called some time after 
; invoking this routine. 
;
; Inputs: 
;   A = Desired position.
;   X = Desired object to be positioned (0-5).
; scanlines: 
;   If control comes on or before cycle 73 then 1 scanline is consumed.
;   If control comes after cycle 73 then 2 scanlines are consumed.
; Outputs: 
;   X = unchanged
;   A = Fine Adjustment value.
;   Y = the "remainder" of the division by 15 minus an additional 15.
;       control is returned on cycle 6 of the next scanline.
;
; TIME: 85 + whatever time was left in the current line

do_sprite_move
    sta WSYNC           ; 3, ?
    sec                 ; 2, 2
DivideLoop              ; This loop MAX 54 cycles if A is < 160, MIN 4
    sbc #15             ; 2, [4
    bcs DivideLoop      ; 2/3 [6,56]        
    tay                 ; 2, [8,58]
    lda FineAdjustTableEnd,Y    ; 5, [13,63]
    nop                 ; 2, [15,65]
    sta HMP0,X          ; 4  [19,69]
    sta RESP0,x         ; 4  [23,73]
    sta WSYNC           ; 3  [26,76]
    sta HMOVE           ; 3, 3
    rts                 ; 6, 9



;
; Input:
;     A : least significat byte of vx position to convert
;     vxoffsetlo : least significant byte of scroll offset 
; Output:
;     A : xposition or $FF if would be off screen

convert_virtual_xpos
    sec                 ; 2, 2
    sbc vxoffsetlo      ; 3, 5
    cmp #1              ; 2, 7
    bcc off_screen      ; 2, 9
    cmp #160            ; 2, 11
    bcs off_screen      ; 2, 13
    rts                 ; 6, 19
off_screen              ; +1, 14
    lda #$FF            ; 2, 16
    rts                 ; 6, 22
convert_virtual_xpos_end

    ORG $FA00

; This table converts the "remainder" of the division by 15 (-1 to -15) to the correct
; fine adjustment value. This table is on a page boundary to guarantee the processor
; will cross a page boundary and waste a cycle in order to be at the precise position
; for a RESP0,x write

FineAdjustTableBegin
        .byte %01100000 ;left 6
        .byte %01010000
        .byte %01000000
        .byte %00110000
        .byte %00100000
        .byte %00010000
        .byte %00000000 ;left/right 0
        .byte %11110000
        .byte %11100000
        .byte %11010000
        .byte %11000000
        .byte %10110000
        .byte %10100000
        .byte %10010000
        .byte %10000000 ;right 8
FineAdjustTableEnd      =       FineAdjustTableBegin - 241

map_debris_table
    .byte #%10000000
    .byte #%01000000
    .byte #%00100000
    .byte #%00010000
    .byte #%00001000
    .byte #%00000100
    .byte #%00000010
    .byte #%00000001
    .byte #%10000000
    .byte #%01000000
    .byte #%00100000
    .byte #%00010000
    .byte #%00001000
    .byte #%00000100
    .byte #%00000010
    .byte #%00000001
    .byte #%10000000
    .byte #%01000000
    .byte #%00100000
    .byte #%00010000
    .byte #%00001000
    .byte #%00000100
    .byte #%00000010
    .byte #%00000001
    .byte #%10000000
    .byte #%01000000
    .byte #%00100000
    .byte #%00010000
    .byte #%00001000
    .byte #%00000100
    .byte #%00000010
    .byte #%00000001

; offset 48

andTable
    .byte #%00000000  ; 000
    .byte #%00000011  ; 001
    .byte #%00000000  ; 010
    .byte #%00000001  ; 011
    .byte #%00000011  ; 100
    .byte #%00000000  ; 101
    .byte #%00000001  ; 110
    .byte #%00000000  ; 111

moveTable
    .byte #$00 
    .byte #1
    .byte #0
    .byte #1
    .byte #255
    .byte #0
    .byte #255
    .byte #0

radarBitmask
    .byte #%10000000
    .byte #%01000000
    .byte #%00100000
    .byte #%00010000
    .byte #%00001000
    .byte #%00000100
    .byte #%00000010
    .byte #%00000001

leftSmartBombTable
    .byte #<leftSmartBomb2
    .byte #<leftSmartBomb2
    .byte #<leftSmartBomb1
    .byte #<emptySmartBomb
    .byte #>leftSmartBomb2
    .byte #>leftSmartBomb2
    .byte #>leftSmartBomb1
    .byte #>emptySmartBomb

rightSmartBombTable
    .byte #<rightSmartBomb2
    .byte #<rightSmartBomb1
    .byte #<emptySmartBomb
    .byte #<emptySmartBomb
    .byte #>rightSmartBomb2
    .byte #>rightSmartBomb1
    .byte #>emptySmartBomb
    .byte #>emptySmartBomb

emptySmartBomb
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
leftSmartBomb1
    .byte #%11100000
    .byte #%01110000
    .byte #%11100000
leftSmartBomb2
    .byte #%11100011
    .byte #%01110001
    .byte #%11100011
rightSmartBomb1
    .byte #%10000000
    .byte #%11000000
    .byte #%10000000
rightSmartBomb2
    .byte #%10001110
    .byte #%11000111
    .byte #%10001110

PlayerSprite
    .byte #%01000010
    .byte #%10100101
    .byte #%01000010
    .byte #%00111100
    .byte #%01111110
    .byte #%01111110
    .byte #%11111111
    .byte #%11111111
    .byte #%00011000
    .byte #%00011000
    .byte #%00011000

PlayerColor
    .byte #$0a
    .byte #$0a
    .byte #$0a
    .byte #$0a
    .byte #$0a
    .byte #$0a
    .byte #$0c
    .byte #$0e
    .byte #$02
    .byte #$42
    .byte #$42

spriteTable
    .byte #<sprite1FrameTable  ; 000 Down (Radioactive)
    .byte #>sprite1FrameTable      
    .byte #<sprite7FrameTable  ; 001 Rightsprite4Frame
    .byte #>sprite7FrameTable  
    .byte #<sprite2FrameTable  ; 010 Down
    .byte #>sprite2FrameTable  
    .byte #<sprite5FrameTable  ; 011 Right Right
    .byte #>sprite5FrameTable   
    .byte #<sprite6FrameTable  ; 100 Left
    .byte #>sprite6FrameTable     
    .byte #<sprite2FrameTable  ; 101 Down
    .byte #>sprite2FrameTable    
    .byte #<sprite4FrameTable  ; 110 Left Left
    .byte #>sprite4FrameTable    
    .byte #<sprite3FrameTable  ; 111 Down (Absorber) 
    .byte #>sprite3FrameTable

sprite1FrameTable              
    .byte #<sprite1Frame1
    .byte #>sprite1Frame1
    .byte #<sprite1Frame2
    .byte #>sprite1Frame2
    .byte #<sprite1Frame3
    .byte #>sprite1Frame3
    .byte #<sprite1Frame4
    .byte #>sprite1Frame4
    .byte #<sprite1Frame5
    .byte #>sprite1Frame5
    .byte #<sprite1Frame6
    .byte #>sprite1Frame6
    .byte #<sprite1Frame7
    .byte #>sprite1Frame7
    .byte #<sprite1Frame8
    .byte #>sprite1Frame8

sprite2FrameTable
    .byte #<sprite2Frame1
    .byte #>sprite2Frame1
    .byte #<sprite2Frame1
    .byte #>sprite2Frame1
    .byte #<sprite2Frame2
    .byte #>sprite2Frame2
    .byte #<sprite2Frame2
    .byte #>sprite2Frame2
    .byte #<sprite2Frame1
    .byte #>sprite2Frame1
    .byte #<sprite2Frame1
    .byte #>sprite2Frame1
    .byte #<sprite2Frame2
    .byte #>sprite2Frame2
    .byte #<sprite2Frame2
    .byte #>sprite2Frame2

sprite3FrameTable
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1
    .byte #<sprite3Frame1
    .byte #>sprite3Frame1

sprite4FrameTable           ; left left
    .byte #<sprite4Frame1
    .byte #>sprite4Frame1
    .byte #<sprite4Frame1
    .byte #>sprite4Frame1
    .byte #<sprite4Frame2
    .byte #>sprite4Frame2
    .byte #<sprite4Frame2
    .byte #>sprite4Frame2
    .byte #<sprite4Frame3
    .byte #>sprite4Frame3
    .byte #<sprite4Frame3
    .byte #>sprite4Frame3
    .byte #<sprite4Frame2
    .byte #>sprite4Frame2
    .byte #<sprite4Frame2
    .byte #>sprite4Frame2

sprite5FrameTable           ; right right
    .byte #<sprite5Frame1
    .byte #>sprite5Frame1
    .byte #<sprite5Frame1
    .byte #>sprite5Frame1
    .byte #<sprite5Frame2
    .byte #>sprite5Frame2
    .byte #<sprite5Frame2
    .byte #>sprite5Frame2
    .byte #<sprite5Frame3
    .byte #>sprite5Frame3
    .byte #<sprite5Frame3
    .byte #>sprite5Frame3
    .byte #<sprite5Frame2
    .byte #>sprite5Frame2
    .byte #<sprite5Frame2
    .byte #>sprite5Frame2

sprite6FrameTable
    .byte #<sprite6Frame1
    .byte #>sprite6Frame1
    .byte #<sprite6Frame1
    .byte #>sprite6Frame1
    .byte #<sprite6Frame2
    .byte #>sprite6Frame2
    .byte #<sprite6Frame2
    .byte #>sprite6Frame2
    .byte #<sprite6Frame1
    .byte #>sprite6Frame1
    .byte #<sprite6Frame1
    .byte #>sprite6Frame1
    .byte #<sprite6Frame2
    .byte #>sprite6Frame2
    .byte #<sprite6Frame2
    .byte #>sprite6Frame2

sprite7FrameTable
    .byte #<sprite7Frame1
    .byte #>sprite7Frame1
    .byte #<sprite7Frame1
    .byte #>sprite7Frame1
    .byte #<sprite7Frame2
    .byte #>sprite7Frame2
    .byte #<sprite7Frame2
    .byte #>sprite7Frame2
    .byte #<sprite7Frame1
    .byte #>sprite7Frame1
    .byte #<sprite7Frame1
    .byte #>sprite7Frame1
    .byte #<sprite7Frame2
    .byte #>sprite7Frame2
    .byte #<sprite7Frame2
    .byte #>sprite7Frame2

explosionFrameTable
    .byte #<explosionSprite1
    .byte #>explosionSprite1
    .byte #<explosionSprite1
    .byte #>explosionSprite1
    .byte #<explosionSprite1
    .byte #>explosionSprite1
    .byte #<explosionSprite1
    .byte #>explosionSprite1
    .byte #<explosionSprite2
    .byte #>explosionSprite2
    .byte #<explosionSprite2
    .byte #>explosionSprite2
    .byte #<explosionSprite2
    .byte #>explosionSprite2
    .byte #<explosionSprite2
    .byte #>explosionSprite2

sprite1Frame1
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors1
    .byte #>boltColors1
sprite1Frame2
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors2
    .byte #>boltColors2
sprite1Frame3
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors3
    .byte #>boltColors3
sprite1Frame4
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors4
    .byte #>boltColors4
sprite1Frame5
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors5
    .byte #>boltColors5
sprite1Frame6
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors6
    .byte #>boltColors6
sprite1Frame7
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors7
    .byte #>boltColors7
sprite1Frame8
        .byte #%00000000
        .byte #%00000000
        .byte #%00000001
        .byte #%00000010
        .byte #%00000110
        .byte #%00001100
        .byte #%00011100
        .byte #%00111000
        .byte #%01111100
        .byte #%00111110
        .byte #%00011100
        .byte #%00111000
        .byte #%00110000
        .byte #%01100000
        .byte #%01000000
        .byte #%10000000
        .byte #%00000000
    .byte #<boltColors8
    .byte #>boltColors8

sprite2Frame1
    .byte #%00000000
    .byte #%00111000
    .byte #%01100110
    .byte #%11111011
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%01111111
    .byte #%01111110
    .byte #%00011100
    .byte #%10000001
    .byte #%0100100
    .byte #%01000100
    .byte #%00010000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1

sprite2Frame2
    .byte #%00000000
    .byte #%00111000
    .byte #%01100110
    .byte #%11111011
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%01111111
    .byte #%01111110
    .byte #%00011100
    .byte #%00000000
    .byte #%00100100
    .byte #%00010000
    .byte #%00100100
    .byte #%00010000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1

sprite2Frame3
    .byte #%00000000
    .byte #%01111110
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%01111110
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1

sprite2Frame4
    .byte #%00000000
    .byte #%01111110
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%01111110
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1

sprite3Frame1
    .byte #%00000000
    .byte #%01111100
    .byte #%11111110
    .byte #%01111100
    .byte #%01111100
    .byte #%00111000
    .byte #%00111000
    .byte #%00010000
    .byte #%00111000
    .byte #%00111000
    .byte #%01111100
    .byte #%01111100
    .byte #%11111110
    .byte #%01111100
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<absorberColor1
    .byte #>absorberColor1

sprite4Frame1
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%01110000
    .byte #%01011000
    .byte #%11111000
    .byte #%11111000
    .byte #%11101000
    .byte #%01110000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor2
    .byte #>DebrisColor2

sprite4Frame2
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%01110000
    .byte #%01011000
    .byte #%11111000
    .byte #%11111000
    .byte #%11101000
    .byte #%01110000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor2
    .byte #>DebrisColor2

sprite4Frame3
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%01110000
    .byte #%01011000
    .byte #%11111000
    .byte #%11111000
    .byte #%11101000
    .byte #%01110000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor2
    .byte #>DebrisColor2


sprite5Frame1
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00001110
    .byte #%00001011
    .byte #%00011111
    .byte #%00011111
    .byte #%00011101
    .byte #%00001110
    .byte #%01010000
    .byte #%00000000
    .byte #%10000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor2
    .byte #>DebrisColor2

sprite5Frame2
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00001110
    .byte #%00001011
    .byte #%00011111
    .byte #%00011111
    .byte #%00011101
    .byte #%00001110
    .byte #%10100000
    .byte #%00000000
    .byte #%01000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor2
    .byte #>DebrisColor2

sprite5Frame3
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00001110
    .byte #%00001011
    .byte #%00011111
    .byte #%00011111
    .byte #%00011101
    .byte #%00001110
    .byte #%01000000
    .byte #%00000000
    .byte #%10000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor2
    .byte #>DebrisColor2

sprite6Frame1
    .byte #%00000000
    .byte #%00000000
    .byte #%01110000
    .byte #%11011000
    .byte #%10111000
    .byte #%11111000
    .byte #%11111000
    .byte #%11111010
    .byte #%11111000
    .byte #%11111010
    .byte #%01110010
    .byte #%00000100
    .byte #%01011001
    .byte #%00000010
    .byte #%00001000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1

sprite6Frame2
    .byte #%00000000
    .byte #%00000000
    .byte #%01110000
    .byte #%11011000
    .byte #%10111000
    .byte #%11111000
    .byte #%11111000
    .byte #%11111000
    .byte #%11111001
    .byte #%11111000
    .byte #%01110001
    .byte #%00000001
    .byte #%00000010
    .byte #%00101100
    .byte #%00000001
    .byte #%00000100
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1

sprite7Frame1
    .byte #%00000000
    .byte #%00000000
    .byte #%00001110
    .byte #%00011011
    .byte #%00011101
    .byte #%00011111
    .byte #%00010111
    .byte #%01011111
    .byte #%00011111
    .byte #%01011111
    .byte #%01001110
    .byte #%00100000
    .byte #%10011010
    .byte #%01000000
    .byte #%00010000
    .byte #%00000000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1



sprite7Frame2
    .byte #%00000000
    .byte #%00000000
    .byte #%00001110
    .byte #%00011011
    .byte #%00011101
    .byte #%00011111
    .byte #%00010111
    .byte #%00011111
    .byte #%10011111
    .byte #%00011111
    .byte #%10001110
    .byte #%10000000
    .byte #%01000000
    .byte #%00110100
    .byte #%10000000
    .byte #%00100000
    .byte #%00000000
    .byte #<DebrisColor1
    .byte #>DebrisColor1


absorberColor1
    .byte #$0e
    .byte #$98
    .byte #$90
    .byte #$9a
    .byte #$90
    .byte #$9c
    .byte #$90
    .byte #$9e
    .byte #$90
    .byte #$9c
    .byte #$90
    .byte #$9a
    .byte #$90
    .byte #$98
    .byte #$90
    .byte #$98
    .byte #$90

DebrisColor1
    .byte #$0e
    .byte #$02
    .byte #$02
    .byte #$02
    .byte #$04
    .byte #$06
    .byte #$08
    .byte #$0a
    .byte #$0e
    .byte #$0c
    .byte #$0a
    .byte #$08
    .byte #$06
    .byte #$04
    .byte #$02
    .byte #$02
    .byte #$02

DebrisColor2
    .byte #$0e
    .byte #$02
    .byte #$02
    .byte #$02
    .byte #$04
    .byte #$06
    .byte #$08
    .byte #$08
    .byte #$0a
    .byte #$3a
    .byte #$3e
    .byte #$3c
    .byte #$3a
    .byte #$38
    .byte #$36
    .byte #$34
    .byte #$32

boltColors8
    .byte #$0e
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
    .byte #$14
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
boltColors7
    .byte #$0e
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
    .byte #$14
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
boltColors6
    .byte #$0e
    .byte #$14
    .byte #$16
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
    .byte #$14
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
boltColors5
    .byte #$0e
    .byte #$12
    .byte #$12
    .byte #$14
    .byte #$16
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
    .byte #$14
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
boltColors4
    .byte #$0e
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$14
    .byte #$16
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
    .byte #$14
    .byte #$12
    .byte #$12
    .byte #$12
boltColors3
    .byte #$0e
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$14
    .byte #$16
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
    .byte #$14
    .byte #$12
boltColors2
    .byte #$0e
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$14
    .byte #$16
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a
    .byte #$18
    .byte #$16
boltColors1
    .byte #$0e
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$12
    .byte #$14
    .byte #$16
    .byte #$18
    .byte #$1a
    .byte #$1e
    .byte #$1a

explosionSprite1
        .byte #%00000000
        .byte #%00000000
        .byte #%00101000
        .byte #%10000100
        .byte #%00101000
        .byte #%10010010
        .byte #%00101001
        .byte #%01000000
        .byte #%10001001
        .byte #%00100000
        .byte #%10001010
        .byte #%01001001
        .byte #%00000100
        .byte #%00100000
        .byte #%01001000
        .byte #%00010000
        .byte #%00000000
    .byte #<explosionColors
    .byte #>explosionColors

explosionSprite2
        .byte #%00000000
        .byte #%00000000
        .byte #%00101000
        .byte #%10010010
        .byte #%01000000
        .byte #%10001001
        .byte #%00100000
        .byte #%00101001
        .byte #%10001010
        .byte #%01001001
        .byte #%00000100
        .byte #%00100000
        .byte #%01001001
        .byte #%00000100
        .byte #%00010000
        .byte #%00100000
        .byte #%00000000
    .byte #<explosionColors
    .byte #>explosionColors

explosionColors
    .byte #$0e
    .byte #$02
    .byte #$08
    .byte #$0a
    .byte #$0a
    .byte #$0e
    .byte #$0e
    .byte #$0e
    .byte #$0e
    .byte #$0e
    .byte #$0e
    .byte #$0e
    .byte #$0e
    .byte #$0a
    .byte #$0a
    .byte #$08
    .byte #$08

scoreTable
    .byte #<scoreZero
    .byte #<scoreOne
    .byte #<scoreTwo
    .byte #<scoreThree
    .byte #<scoreFour
    .byte #<scoreFive
    .byte #<scoreSix
    .byte #<scoreSeven
    .byte #<scoreEight
    .byte #<scoreNine
    .byte #>scoreZero
    .byte #>scoreOne
    .byte #>scoreTwo
    .byte #>scoreThree
    .byte #>scoreFour
    .byte #>scoreFive
    .byte #>scoreSix
    .byte #>scoreSeven
    .byte #>scoreEight
    .byte #>scoreNine

scoreZero
   .byte #%00111000
   .byte #%01000100
   .byte #%01000100
   .byte #%01000100
   .byte #%01000100
   .byte #%01000100
   .byte #%00111000
scoreOne
   .byte #%00111000
   .byte #%00010000
   .byte #%00010000
   .byte #%00010000
   .byte #%00010000
   .byte #%00110000
   .byte #%00010000
scoreTwo
   .byte #%01111100
   .byte #%00100000
   .byte #%00010000
   .byte #%00001000
   .byte #%00000100
   .byte #%01000100
   .byte #%00111000
scoreThree
   .byte #%00111000
   .byte #%01000100
   .byte #%00000100
   .byte #%00111000
   .byte #%00000100
   .byte #%01000100
   .byte #%00111000
scoreFour
   .byte #%00001000
   .byte #%00001000
   .byte #%00001000
   .byte #%01111100
   .byte #%01001000
   .byte #%01001000
   .byte #%01001000
scoreFive
   .byte #%00111000
   .byte #%01000100
   .byte #%00000100
   .byte #%00000100
   .byte #%01111000
   .byte #%01000000
   .byte #%01111100
scoreSix
   .byte #%00111000
   .byte #%01000100
   .byte #%01000100
   .byte #%01111000
   .byte #%01000000
   .byte #%01000100
   .byte #%00111000
scoreSeven
   .byte #%00100000
   .byte #%00100000
   .byte #%00010000
   .byte #%00001000
   .byte #%00000100
   .byte #%00000100
   .byte #%01111000
scoreEight
   .byte #%00111000
   .byte #%01000100
   .byte #%01000100
   .byte #%00111000
   .byte #%01000100
   .byte #%01000100
   .byte #%00111000
scoreNine
   .byte #%00111000
   .byte #%01000100
   .byte #%00000100
   .byte #%00111100
   .byte #%01000100
   .byte #%01000100
   .byte #%00111000

debrisMoveTable
    ; speed 1
    .byte #$1
    .byte #$1
    .byte #$1
    .byte #$1
    ; speed 2
    .byte #$1
    .byte #$1
    .byte #$1
    .byte #$2
    ; speed 3
    .byte #$1
    .byte #$2
    .byte #$1
    .byte #$2
    ; speed 4
    .byte #$1
    .byte #$1
    .byte #$1
    .byte #$1
    ; speed 5
    .byte #$1
    .byte #$2
    .byte #$1
    .byte #$2
    ; speed 6
    .byte #$1
    .byte #$1
    .byte #$1
    .byte #$2
    ; speed 7
    .byte #$1
    .byte #$2
    .byte #$1
    .byte #$2
    ; speed 8
    .byte #$1
    .byte #$2
    .byte #$2
    .byte #$2

radarColorTable
    .byte #$1c  ; radioactive debris
    .byte #$0c  ; inert debris
    .byte #$0c  ; inert debris
    .byte #$0c  ; inert debris
    .byte #$0c  ; inert debris
    .byte #$0c  ; inert debris
    .byte #$0c  ; inert debris
    .byte #$9c  ; absorbing debris

    include "pfdata.asm"



;
; The 2600 looks at this memory location to find the program entry point
; 

    ORG $FFFA
    .word sysinit          ; NMI
    .word sysinit          ; RESET
    .word sysinit          ; IRQ 

END

;
; Each sprite has the following overhead:
;     frame table      : 16 bytes
;     color table      : 16 bytes
;     each frame       : 17 bytes
;     each color frame : 17 bytes
;     at least one entry in the index : 2 bytes
; So, a sprite with 8 full color frames takes 16 + 16 + 2 + 17x8 + 17x8 = 306 bytes
; A sprite with 2 full color frames = 102 bytes
; A sprite with 1 frame = 68 bytes
;





